-- Lesson: Room with heating system + Cyclical external environment temperature - Three charts
-- Author: Tiago Carneiro

--------------------------------------------------------------
-- PARAMETER
--------------------------------------------------------------
THEMORSTAT_VALUE			= 30			-- 5 	25	(30)	50	100	1000	
HEAT_EXCHANGE_RATE 		= 0.1			-- 0	(0.1)	0.5	1
FURNANCE_TEMPERATURE 	= 30			--	(30)	100
FINAL_TIME 						= 60 			-- 5	(60)
DT 									= 0.03125 	-- try	1		0.5		0.25 		0.125 		0.0625		(0.03125)
--------------------------------------------------------------
-- MODEL
--------------------------------------------------------------

function simulateSeasonalTemperature(t, amplitude, periodicity, translationY, translationX)
	return (amplitude/2) * math.sin(2*math.pi*(1/periodicity)*t + translationX) + translationY 
end


function change(tempPast, externalTemp )
	local thermostatDiscrepancy 		= THEMORSTAT_VALUE - tempPast
	local heatingRate 					=  thermostatDiscrepancy / THEMORSTAT_VALUE
	local environmentDiscrepancy	= tempPast - externalTemp
	return  FURNANCE_TEMPERATURE * heatingRate -  HEAT_EXCHANGE_RATE * environmentDiscrepancy
end 


system = Agent{
	externalTemp 	= 17,
	roomTemp  	= 20,
	exchangedHeat = 0,
	execute		= function( self, t, amplitude, periodicity, translationY, translationX )
		self.externalTemp	= simulateSeasonalTemperature(t, amplitude, periodicity, translationY, translationX)
		self.exchangedHeat	= change( self.roomTemp, self.externalTemp )
		self.roomTemp 		= self.roomTemp + self.exchangedHeat * DT
	end
}

--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

obs1 = Observer{subject = system, type = "chart", attributes = {"externalTemp" } , title = "Temperature", curveLabels={"external temp"}}
obs3 = Observer{subject = system, type = "chart", attributes = {"roomTemp" } , title = "Temperature", curveLabels={"room temp"}}
obs3 = Observer{subject = system, type = "chart", attributes = {"exchangedHeat" } , title = "Temperature", curveLabels={"exchanged heat"}}


for t = 0, FINAL_TIME, DT do

	print( t, system.externalTemp, system.roomTemp, system.exchangedHeat  )

	system:execute( t, 7, 15, 16.5, 0)

	system:notify(t)

end